package main

import (
	"net/http"
	redismgr "server/db/redis"
	"server/msg"
	"server/user"

	"github.com/gin-contrib/cors"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"github.com/name5566/leaf/log"
	"google.golang.org/protobuf/proto"
)

func ParseProtoBuf(message proto.Message) gin.HandlerFunc {
	return func(c *gin.Context) {
		// 创建目标消息的新实例
		msg := proto.Clone(message)

		// 解析 Protobuf 数据
		if err := c.ShouldBindBodyWith(msg, binding.ProtoBuf); err != nil {
			c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{
				"error":   "Invalid protobuf data",
				"details": err.Error(),
			})
			return
		}

		// 存储解析后的消息
		c.Set("protobuf_data", msg)
		c.Next()
	}
}

// func ParseProtoBuf(c *gin.Context) {
// 	// 1. 读取原始数据
// 	body, err := io.ReadAll(c.Request.Body)
// 	if err != nil {
// 		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"error": "Failed to read body"})
// 		return
// 	}

// 	// 2. 存储原始二进制数据（供后续处理）
// 	c.Set("protobuf_raw", body)
// 	c.Next()
// }

func main() {
	r := gin.Default()

	// 使用 cors.New 创建中间件，并配置允许所有域的访问
	config := cors.DefaultConfig()
	config.AllowOrigins = []string{"*"} // 允许所有域
	config.AllowMethods = []string{"GET", "POST", "PUT", "PATCH", "DELETE", "HEAD", "OPTIONS"}
	config.AllowHeaders = []string{"Origin", "Content-Length", "Content-Type", "Cookie"}
	config.AllowCredentials = true

	r.Use(cors.New(config))

	// r.GET("/", func(c *gin.Context) {
	// 	c.String(200, "Hello, Gin!")
	// })

	// r.GET("/json", func(c *gin.Context) {
	// 	c.JSON(200, gin.H{"message": "Hello, JSON!"})
	// })

	// r.GET("/test", func(c *gin.Context) {
	// 	c.Redirect(http.StatusMovedPermanently, "http://www.google.com/")
	// })

	r.POST("/ReqLogin", ParseProtoBuf(&msg.ReqLogin{}), ReqLogin)
	r.POST("/ReqRegister", ParseProtoBuf(&msg.ReqRegister{}), ReqRegister)
	r.Run(":8080") // 默认监听 :8080
}

func ReqRegister(c *gin.Context) {
	req := c.MustGet("protobuf_data").(*msg.ReqRegister)

	log.Debug(req.Account, req.Password)

	userId, _ := redismgr.GetUserIDFromRedisByAP(req.Account, req.Password)

	if userId != "" {
		c.ProtoBuf(http.StatusOK, &msg.ResRegister{
			Success: false,
			ErrMsg: &msg.MsgError{
				ErrorCode: 101,
				ErrorMsg:  " Account is exist！",
			},
		})
		return
	}

	userId, Success := redismgr.SetUserIDFromRedisByAP(req.Account, req.Password)
	if !Success {
		c.ProtoBuf(http.StatusOK, &msg.ResRegister{
			Success: false,
			ErrMsg: &msg.MsgError{
				ErrorCode: 102,
				ErrorMsg:  "server registerHandler error",
			},
		})
		return
	}

	redismgr.SaveUserInfoToRedis(&msg.UserInfo{
		UserId: userId,
		MCoin:  0,
		MHead:  req.MHead,
		Name:   req.NikeName,
	})
	c.ProtoBuf(http.StatusOK, &msg.ResRegister{
		Success: true,
		ErrMsg:  nil,
	})
}

func ReqLogin(c *gin.Context) {
	req := c.MustGet("protobuf_data").(*msg.ReqLogin)
	UserId, err := redismgr.GetUserIDFromRedisByAP(req.Account, req.Password)
	if err != nil {

		c.ProtoBuf(http.StatusOK, &msg.ResLogin{
			NikeName: "",
			UserId:   "",
			ErrMsg: &msg.MsgError{
				ErrorCode: 101,
				ErrorMsg:  "Password or Account is error！",
			},
		})
		return
	}
	// 获取用户信息 ,如果新用户则创建一个
	userData := user.GetUserInfoById(UserId)
	c.ProtoBuf(http.StatusOK, &msg.ResLogin{
		UserInfo: &msg.UserInfo{
			UserId: userData.UserId,
			MHead:  userData.MHead,
			MCoin:  userData.MCoin,
			Name:   userData.Name,
			RoomId: userData.RoomId,
		},
		ErrMsg: nil,
	})
}
